@@ -18,21 +18,34 @@
  */
 package org.apache.myfaces.renderkit.html;
 
-import org.apache.myfaces.shared_impl.renderkit.html.HtmlTableRendererBase;
+import javax.faces.context.FacesContext;
+
+import org.apache.myfaces.buildtools.maven2.plugin.builder.annotation.JSFRenderer;
+import org.apache.myfaces.shared.renderkit.html.HtmlTableRendererBase;
 
 
 /**
- * @JSFRenderer
- *   renderKitId="HTML_BASIC"
- *   family="javax.faces.Data"
- *   type="javax.faces.Table"
  * 
- * @author Thomas Spiegl (latest modification by $Author: lu4242 $)
- * @version $Revision: 659857 $ $Date: 2008-05-24 13:17:47 -0500 (Sat, 24 May 2008) $
+ * @author Thomas Spiegl (latest modification by $Author$)
+ * @version $Revision$ $Date$
  */
+@JSFRenderer(
+    renderKitId="HTML_BASIC",
+    family="javax.faces.Data",
+    type="javax.faces.Table")
 public class HtmlTableRenderer
     extends HtmlTableRendererBase
 {
-    //private static final Log log = LogFactory.getLog(HtmlTableRenderer.class);
+    @Override
+    protected boolean isCommonPropertiesOptimizationEnabled(FacesContext facesContext)
+    {
+        return true;
+    }
+
+    @Override
+    protected boolean isCommonEventsOptimizationEnabled(FacesContext facesContext)
+    {
+        return true;
+    }
 
 }
