@@ -23,20 +23,16 @@ import java.util.Map;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 
-import javax.faces.application.ProjectStage;
 import javax.faces.application.ViewHandler;
 import javax.faces.context.ExternalContext;
 import javax.faces.context.FacesContext;
 import javax.faces.render.ResponseStateManager;
 import javax.faces.view.ViewDeclarationLanguage;
 
-import org.apache.myfaces.buildtools.maven2.plugin.builder.annotation.JSFWebConfigParam;
 import org.apache.myfaces.shared.renderkit.html.util.SharedStringBuilder;
-import org.apache.myfaces.shared.util.ConcurrentLRUCache;
 import org.apache.myfaces.shared.util.ExternalContextUtils;
 import org.apache.myfaces.shared.util.StringUtils;
 import org.apache.myfaces.shared.util.ViewProtectionUtils;
-import org.apache.myfaces.shared.util.WebConfigParamUtils;
 
 /**
  * A ViewHandlerSupport implementation for use with standard Java Servlet engines,
@@ -52,37 +48,14 @@ public class DefaultViewHandlerSupport implements ViewHandlerSupport
 
     //private static final Log log = LogFactory.getLog(DefaultViewHandlerSupport.class);
     private static final Logger log = Logger.getLogger(DefaultViewHandlerSupport.class.getName());
-
-    /**
-     * Controls the size of the cache used to "remember" if a view exists or not.
-     */
-    @JSFWebConfigParam(defaultValue = "500", since = "2.0.2", group="viewhandler", tags="performance", 
-            classType="java.lang.Integer",
-            desc="Controls the size of the cache used to 'remember' if a view exists or not.")
-    private static final String CHECKED_VIEWID_CACHE_SIZE_ATTRIBUTE = "org.apache.myfaces.CHECKED_VIEWID_CACHE_SIZE";
-    private static final int CHECKED_VIEWID_CACHE_DEFAULT_SIZE = 500;
-
-    /**
-     * Enable or disable a cache used to "remember" if a view exists or not and reduce the impact of
-     * sucesive calls to ExternalContext.getResource().
-     */
-    @JSFWebConfigParam(defaultValue = "true", since = "2.0.2", expectedValues="true, false", group="viewhandler", 
-            tags="performance",
-            desc="Enable or disable a cache used to 'remember' if a view exists or not and reduce the impact " +
-                 "of sucesive calls to ExternalContext.getResource().")
-    private static final String CHECKED_VIEWID_CACHE_ENABLED_ATTRIBUTE = 
-        "org.apache.myfaces.CHECKED_VIEWID_CACHE_ENABLED";
-    private static final boolean CHECKED_VIEWID_CACHE_ENABLED_DEFAULT = true;
     
     private static final String VIEW_HANDLER_SUPPORT_SB = "oam.viewhandler.SUPPORT_SB";
-
-    private volatile ConcurrentLRUCache<String, Boolean> _checkedViewIdMap = null;
-    private Boolean _checkedViewIdCacheEnabled = null;
     
     private final String[] _faceletsViewMappings;
     private final String[] _contextSuffixes;
     private final String _faceletsContextSufix;
     private final boolean _initialized;
+    private CheckedViewIdsCache checkedViewIdsCache = null;
     
     public DefaultViewHandlerSupport()
     {
@@ -112,6 +85,14 @@ public class DefaultViewHandlerSupport implements ViewHandlerSupport
         {
             viewId = handleSuffixMapping(context, viewId);
         }
+        else if (mapping.isExactMapping())
+        {
+            // if the current request is a exact mapping and the viewId equals the exact viewId
+            if (viewId.equals(mapping.getExact()))
+            {
+                viewId = handleSuffixMapping(context, viewId + ".jsf");
+            }
+        }
         else if(mapping.isPrefixMapping())
         {
             viewId = handlePrefixMapping(viewId,mapping.getPrefix());
@@ -125,6 +106,14 @@ public class DefaultViewHandlerSupport implements ViewHandlerSupport
             {
                 throw new InvalidViewIdException();
             }
+            
+            // In JSF 2.3 some changes were done in the VDL to avoid the jsp vdl
+            // RequestDispatcher redirection (only accept viewIds with jsp extension).
+            // If we have this case
+            if (viewId != null && viewId.equals(mapping.getPrefix()))
+            {
+                viewId = handleSuffixMapping(context, viewId+".jsf");
+            }
         }
         else if (mapping.getUrlPattern().startsWith(viewId))
         {
@@ -151,6 +140,14 @@ public class DefaultViewHandlerSupport implements ViewHandlerSupport
         {
             viewId = handleSuffixMapping(context, viewId);
         }
+        else if (mapping.isExactMapping())
+        {
+            // if the current request is a exact mapping and the viewId equals the exact viewId
+            if (viewId.equals(mapping.getExact()))
+            {
+                viewId = handleSuffixMapping(context, viewId + ".jsf");
+            }
+        }
         else if(mapping.isPrefixMapping())
         {
             viewId = handlePrefixMapping(viewId,mapping.getPrefix());
@@ -161,10 +158,18 @@ public class DefaultViewHandlerSupport implements ViewHandlerSupport
                 // considered invalid, because jsp vdl will use RequestDispatcher and cause
                 // a loop that ends in a exception. Note in portlet mode the view
                 // could be encoded as a query param, so the viewId could be valid.
-                if (viewId.equals(mapping.getPrefix()) &&
-                    !ExternalContextUtils.isPortlet(context.getExternalContext()))
+                //if (viewId.equals(mapping.getPrefix()) &&
+                //    !ExternalContextUtils.isPortlet(context.getExternalContext()))
+                //{
+                //    throw new InvalidViewIdException();
+                //}
+            
+                // In JSF 2.3 some changes were done in the VDL to avoid the jsp vdl
+                // RequestDispatcher redirection (only accept viewIds with jsp extension).
+                // If we have this case
+                if (viewId != null && viewId.equals(mapping.getPrefix()))
                 {
-                    throw new InvalidViewIdException();
+                    viewId = handleSuffixMapping(context, viewId+".jsf");
                 }
 
                 return (checkResourceExists(context,viewId) ? viewId : null);
@@ -203,9 +208,44 @@ public class DefaultViewHandlerSupport implements ViewHandlerSupport
         {
             builder.append(contextPath);
         }
+        
+        // In JSF 2.3 we could have cases where the viewId can be bound to an url-pattern that is not
+        // prefix or suffix, but exact mapping. In this part we need to take the viewId and check if
+        // the viewId is bound or not with a mapping.
+        if (mapping != null && mapping.isExactMapping())
+        {
+            String exactMappingViewId = calculatePrefixedExactMapping(context, viewId);
+            if (exactMappingViewId != null && !exactMappingViewId.isEmpty())
+            {
+                // if the current exactMapping already matches the requested viewId -> same view, skip....
+                if (!mapping.getExact().equals(exactMappingViewId))
+                {
+                    // different viewId -> lets try to lookup a exact mapping
+                    mapping = FacesServletMappingUtils.getExactMapping(context, exactMappingViewId);
+
+                    // no exactMapping for the requested viewId available BUT the current view is a exactMapping
+                    // we need a to search for a prefix or extension mapping
+                    if (mapping == null)
+                    {
+                        mapping = FacesServletMappingUtils.getGenericPrefixOrSuffixMapping(context);
+                        if (mapping == null)
+                        {
+                            throw new IllegalStateException(
+                                    "No generic (either prefix or suffix) servlet-mapping found for FacesServlet."
+                                    + "This is required serve views, that are not exact mapped.");
+                        }
+                    }
+                }
+            }
+        }
+        
         if (mapping != null)
         {
-            if (mapping.isExtensionMapping())
+            if (mapping.isExactMapping())
+            {
+                builder.append(mapping.getExact());
+            }
+            else if (mapping.isExtensionMapping())
             {
                 //See JSF 2.0 section 7.5.2 
                 String[] contextSuffixes = _initialized ? _contextSuffixes : getContextSuffix(context); 
@@ -249,7 +289,7 @@ public class DefaultViewHandlerSupport implements ViewHandlerSupport
                     }
                 }
             }
-            else
+            else if (mapping.isPrefixMapping())
             {
                 builder.append(mapping.getPrefix());
                 builder.append(viewId);
@@ -285,6 +325,22 @@ public class DefaultViewHandlerSupport implements ViewHandlerSupport
         }
         return calculatedActionURL;
     }
+    
+    private String calculatePrefixedExactMapping(FacesContext context, String viewId)
+    {
+        String[] contextSuffixes = _initialized ? _contextSuffixes : getContextSuffix(context); 
+        String prefixedExactMapping = null;
+        for (String contextSuffix : contextSuffixes)
+        {
+            if (viewId.endsWith(contextSuffix))
+            {
+                prefixedExactMapping = viewId.substring(0, viewId.length() - contextSuffix.length());
+                break;
+            }
+        }
+        return prefixedExactMapping == null ? viewId : prefixedExactMapping;
+    }
+            
 
     /**
      * Read the web.xml file that is in the classpath and parse its internals to
@@ -299,59 +355,16 @@ public class DefaultViewHandlerSupport implements ViewHandlerSupport
         if (mapping == null)
         {
             ExternalContext externalContext = context.getExternalContext();
-            mapping = calculateFacesServletMapping(externalContext.getRequestServletPath(),
-                    externalContext.getRequestPathInfo());
+            mapping = FacesServletMappingUtils.calculateFacesServletMapping(
+                    context, externalContext.getRequestServletPath(),
+                    externalContext.getRequestPathInfo(),
+                    true);
 
             attributes.put(CACHED_SERVLET_MAPPING, mapping);
         }
         return mapping;
     }
 
-    /**
-     * Determines the mapping of the FacesServlet in the web.xml configuration
-     * file. However, there is no need to actually parse this configuration file
-     * as runtime information is sufficient.
-     *
-     * @param servletPath The servletPath of the current request
-     * @param pathInfo    The pathInfo of the current request
-     * @return the mapping of the FacesServlet in the web.xml configuration file
-     */
-    protected static FacesServletMapping calculateFacesServletMapping(
-        String servletPath, String pathInfo)
-    {
-        if (pathInfo != null)
-        {
-            // If there is a "extra path", it's definitely no extension mapping.
-            // Now we just have to determine the path which has been specified
-            // in the url-pattern, but that's easy as it's the same as the
-            // current servletPath. It doesn't even matter if "/*" has been used
-            // as in this case the servletPath is just an empty string according
-            // to the Servlet Specification (SRV 4.4).
-            return FacesServletMapping.createPrefixMapping(servletPath);
-        }
-        else
-        {
-            // In the case of extension mapping, no "extra path" is available.
-            // Still it's possible that prefix-based mapping has been used.
-            // Actually, if there was an exact match no "extra path"
-            // is available (e.g. if the url-pattern is "/faces/*"
-            // and the request-uri is "/context/faces").
-            int slashPos = servletPath.lastIndexOf('/');
-            int extensionPos = servletPath.lastIndexOf('.');
-            if (extensionPos > -1 && extensionPos > slashPos)
-            {
-                String extension = servletPath.substring(extensionPos);
-                return FacesServletMapping.createExtensionMapping(extension);
-            }
-            else
-            {
-                // There is no extension in the given servletPath and therefore
-                // we assume that it's an exact match using prefix-based mapping.
-                return FacesServletMapping.createPrefixMapping(servletPath);
-            }
-        }
-    }
-
     protected String[] getContextSuffix(FacesContext context)
     {
         String defaultSuffix = context.getExternalContext().getInitParameter(ViewHandler.DEFAULT_SUFFIX_PARAM_NAME);
@@ -389,9 +402,9 @@ public class DefaultViewHandlerSupport implements ViewHandlerSupport
     /**
      * Return the normalized viewId according to the algorithm specified in 7.5.2 
      * by stripping off any number of occurrences of the prefix mapping from the viewId.
-     * <p/>
+     * <p>
      * For example, both /faces/view.xhtml and /faces/faces/faces/view.xhtml would both return view.xhtml
-     * F 
+     * </p>
      */
     protected String handlePrefixMapping(String viewId, String prefix)
     {
@@ -447,8 +460,9 @@ public class DefaultViewHandlerSupport implements ViewHandlerSupport
     /**
      * Return the viewId with any non-standard suffix stripped off and replaced with
      * the default suffix configured for the specified context.
-     * <p/>
+     * <p>
      * For example, an input parameter of "/foo.jsf" may return "/foo.jsp".
+     * </p>
      */
     protected String handleSuffixMapping(FacesContext context, String requestViewId)
     {
@@ -556,101 +570,48 @@ public class DefaultViewHandlerSupport implements ViewHandlerSupport
         return null;
     }
     
-    protected boolean checkResourceExists(FacesContext context, String viewId)
+    protected boolean checkResourceExists(FacesContext facesContext, String viewId)
     {
+        if (checkedViewIdsCache == null)
+        {
+            checkedViewIdsCache = CheckedViewIdsCache.getInstance(facesContext);
+        }
+
         try
         {
-            if (isCheckedViewIdCachingEnabled(context))
+            Boolean resourceExists = null;
+            if (checkedViewIdsCache.isEnabled())
             {
-                Boolean resourceExists = getCheckedViewIDMap(context).get(
-                        viewId);
-                if (resourceExists == null)
-                {
-                    ViewDeclarationLanguage vdl = context.getApplication().getViewHandler()
-                            .getViewDeclarationLanguage(context, viewId);
-                    if (vdl != null)
-                    {
-                        resourceExists = vdl.viewExists(context, viewId);
-                    }
-                    else
-                    {
-                        // Fallback to default strategy
-                        resourceExists = context.getExternalContext().getResource(
-                                viewId) != null;
-                    }
-                    getCheckedViewIDMap(context).put(viewId, resourceExists);
-                }
-                return resourceExists;
+                resourceExists = checkedViewIdsCache.getCache().get(viewId);
             }
-            else
+
+            if (resourceExists == null)
             {
-                ViewDeclarationLanguage vdl = context.getApplication().getViewHandler()
-                            .getViewDeclarationLanguage(context, viewId);
+                ViewDeclarationLanguage vdl = facesContext.getApplication().getViewHandler()
+                        .getViewDeclarationLanguage(facesContext, viewId);
                 if (vdl != null)
                 {
-                    if (vdl.viewExists(context, viewId))
-                    {
-                        return true;
-                    }
+                    resourceExists = vdl.viewExists(facesContext, viewId);
                 }
                 else
                 {
                     // Fallback to default strategy
-                    if (context.getExternalContext().getResource(viewId) != null)
-                    {
-                        return true;
-                    }
+                    resourceExists = facesContext.getExternalContext().getResource(viewId) != null;
+                }
+
+                if (checkedViewIdsCache.isEnabled())
+                {
+                    checkedViewIdsCache.getCache().put(viewId, resourceExists);
                 }
             }
+
+            return resourceExists;
         }
-        catch(MalformedURLException e)
+        catch (MalformedURLException e)
         {
             //ignore and move on
         }     
         return false;
     }
 
-    private ConcurrentLRUCache<String, Boolean> getCheckedViewIDMap(FacesContext context)
-    {
-        if (_checkedViewIdMap == null)
-        {
-            int maxSize = getViewIDCacheMaxSize(context);
-            _checkedViewIdMap = new ConcurrentLRUCache<String, Boolean>((maxSize * 4 + 3) / 3, maxSize);
-        }
-        return _checkedViewIdMap;
-    }
-
-    private boolean isCheckedViewIdCachingEnabled(FacesContext context)
-    {
-        if (_checkedViewIdCacheEnabled == null)
-        {
-            // first, check if the ProjectStage is development and skip caching in this case
-            if (context.isProjectStage(ProjectStage.Development))
-            {
-                _checkedViewIdCacheEnabled = Boolean.FALSE;
-            }
-            else
-            {
-                // in all ohter cases, make sure that the cache is not explicitly disabled via context param
-                _checkedViewIdCacheEnabled = WebConfigParamUtils.getBooleanInitParameter(context.getExternalContext(),
-                        CHECKED_VIEWID_CACHE_ENABLED_ATTRIBUTE,
-                        CHECKED_VIEWID_CACHE_ENABLED_DEFAULT);
-            }
-
-            if (log.isLoggable(Level.FINE))
-            {
-                log.log(Level.FINE, "MyFaces ViewID Caching Enabled="
-                        + _checkedViewIdCacheEnabled);
-            }
-        }
-        return _checkedViewIdCacheEnabled;
-    }
-
-    private int getViewIDCacheMaxSize(FacesContext context)
-    {
-        ExternalContext externalContext = context.getExternalContext();
-
-        return WebConfigParamUtils.getIntegerInitParameter(externalContext,
-                CHECKED_VIEWID_CACHE_SIZE_ATTRIBUTE, CHECKED_VIEWID_CACHE_DEFAULT_SIZE);
-    }
 }
