@@ -20,9 +20,6 @@ package org.apache.myfaces.shared.resource;
 
 /**
  * Contains the metadata information to reference a resource 
- * 
- * @author Leonardo Uribe (latest modification by $Author: bommel $)
- * @version $Revision: 1187701 $ $Date: 2011-10-22 12:21:54 +0000 (Sat, 22 Oct 2011) $
  */
 public abstract class ResourceMeta
 {
@@ -40,4 +37,13 @@ public abstract class ResourceMeta
     public abstract String getResourceIdentifier();
     
     public abstract boolean couldResourceContainValueExpressions();
+    
+    /**
+     * @since 2.2
+     * @return 
+     */
+    public String getContractName()
+    {
+        return null;
+    }
 }
