@@ -19,11 +19,9 @@
 package javax.faces.event;
 
 import javax.faces.component.UIComponent;
+import javax.faces.context.FacesContext;
 
 /**
- * @author Simon Lessard (latest modification by $Author$)
- * @version $Revision$ $Date$
- *
  * @since 2.0
  */
 public class PostAddToViewEvent extends ComponentSystemEvent
@@ -35,6 +33,14 @@ public class PostAddToViewEvent extends ComponentSystemEvent
     {
         super(component);
     }
+    
+    /**
+     * @since 2.3
+     */
+    public PostAddToViewEvent(FacesContext facesContext, UIComponent component)
+    {
+        super(facesContext, component);
+    }
 
     /**
      * {@inheritDoc}
